
/*
 *  Copyright 2011 Anyware Services
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 */

Ext.namespace('org.ametys.cms.editor.Editor');

/**
 * @constant
 * @type {boolean}
 * Is tiny mce initiated ?
 */
org.ametys.cms.editor.Editor._tinyMCEInitiated = false;

/**
 * @private
 * @static
 * @property {integer} _suspended The number of times the transmission was suspended. 0 means transmission of selection events between tinymce and the ribbon are not suspended. Cannot be negative.
 */
org.ametys.cms.editor.Editor._suspended;

/**
 * @private
 */
org.ametys.cms.editor.Editor._tags = {};
/**
 * This method retrieve a tag to handle.
 * Just by calling this methodd, the tag will be handle by the editor.
 * Once you have a tag it a map
 * - emptyTag Can be empty, +, - or #. see tinymce valid_elements documentation
 * - getAttributes Call this method to handle an attribute. This will return a map where key is the attribute name and value are
 *          - values An array of possible values
 *          - defaultValue The default value. Can be null if there is no
 * - replaceTags An array of tags that will be replaced by this one. This value can be dynamically removed if some one register this other tag as handled.
 * 
 * e.g. var aTag = org.ametys.cms.editor.Editor.handleTag("a"); // from now <a> will be accepted
 *      aTag.emptyTag = "+"; // The <a> tag will be forced opened
 *      var classAttr = aTag.handleAttribute("class"); // The class attribute is now accepted on the <a> tag
 *      classAttr.defaultValue = "myclass"; // The class attribute will now always exists on <a> tag with the value "myclass"
 *      classAttr.values.push("myclass");
 *      classAttr.values.push("myclass2"); // The class attribute will now accept 2 values myclass or myclass2
 */
org.ametys.cms.editor.Editor.handleTag = function(tagName)
{
	if (org.ametys.cms.editor.Editor._tags[tagName] == null)
	{
		org.ametys.cms.editor.Editor._tags[tagName] = {
				tagName: tagName,
				emptyTag: "",
				attributes: {},
				handleAttribute: function(attributeName)
				{
					if (this.attributes[attributeName] == null)
					{
						this.attributes[attributeName] = {
							attributeName: attributeName,
							values: [],
							defaultValue: null,
							toString: function() {
								var a = this.defaultValue != null ? this.attributeName + "=" + this.defaultValue : "";

								var b = ""
								for (var key in this.values)
								{
									if (typeof this.values[key] == "string")
									{
										if (b != "") 
										{
											b += "?";
										}
																		
										b += this.values[key];
									}
								}
								if (b != "")
								{
									b = this.attributeName + "<" + b;
								}
								
								if (a == "" && b == "")
								{
									return this.attributeName;
								}
								else 
								{
									return a + ((a != "" && b != "") ? "|" : "") + b;
								}
							}
						};
					}
					return this.attributes[attributeName];
				},
				replaceTags: [],
				toString: function() {
					var base = this.emptyTag + tagName;
					
					var attributes = "";
					for (var key in this.attributes)
					{
						if (attributes != "")
						{
							attributes += "|"; 
						}
						attributes += this.attributes[key].toString(); 
					}
					
					var attrs = (attributes == "" ? "" : "[" + attributes + "]");
					
					if (this.replaceTags.length == 0)
					{
						return base + attrs;
					}
					else
					{
						var finals = "";
						for (var i = 0; i < this.replaceTags.length; i++)
						{
							// The second part of the condition allow to remove multiple instance of the same string
							if (org.ametys.cms.editor.Editor._tags[this.replaceTags[i]] == null && this.replaceTags.indexOf(this.replaceTags[i]) == i)
							{
								finals += base + "/" + this.replaceTags[i] + attrs + ",";
							}
						}
						return finals.substring(0, finals.length - 1); 
					}
					
				}
		};
	}
	return org.ametys.cms.editor.Editor._tags[tagName];
}
org.ametys.cms.editor.Editor.handleTag("p").emptyTag = "#";
org.ametys.cms.editor.Editor.handleTag("br");

/**
 * A list of css files to load in the inline editor
 */
org.ametys.cms.editor.Editor.useCSSFile = [];
org.ametys.cms.editor.Editor.useCSSFile.push(getPluginResourcesUrl('cms') + "/css/content/ContentToolEditor.css");

/**
 * Storing conversion methods
 * onSetContent is an array of method with signature (editor, object) that is called when the editor received new content. This allows to convert storing tags to internal tags. Use object.content to get/set the full html.
 * onGetContent is an array of method with signature (editor, object) that is called when the editor received content. This allows to convert internat tags to storing tags. Use object.content to get/set the full html.
 * onKeyPress is an array of method with signature (editor, e) that is called when the editor has a key down.
 * onKeyDown is an array of method with signature (editor, e) that is called when the editor has a key down.
 * onKeyUp is an array of method with signature (editor, e) that is called when the editor has a key up.
 * onVisualAid is an array of method with signature (editor, object) that is called when the editor pre process the serialization.
 */
org.ametys.cms.editor.Editor.onSetContent = [];
org.ametys.cms.editor.Editor.onGetContent = [];
org.ametys.cms.editor.Editor.onKeyPress = [];
org.ametys.cms.editor.Editor.onKeyDown = [];
org.ametys.cms.editor.Editor.onKeyUp = [];
org.ametys.cms.editor.Editor.onVisualAid = [];
org.ametys.cms.editor.Editor.onPreProcess = [];

// BUG FIX CMS 2499
if (tinymce.isGecko)
{
	org.ametys.cms.editor.Editor.onKeyPress.push(
		function(ed, e)
		{
			if (e.keyCode == 13 || e.charCode == 13) 
			{
				var rng = ed.selection.getRng();
	
				if (rng.startContainer && ed.getBody() == rng.startContainer && /^table$/i.test(ed.selection.getStart().nodeName)) 
				{
					ed.execCommand('mceInsertContent', false, '<p><br mce_bogus="1"/></p>');
	
					return tinymce.dom.Event.cancel(e);
				}
			}
		}
	);
	org.ametys.cms.editor.Editor.onKeyDown.push(
		/** UPGRADE (tinyMCE 3.3.9.1) plugins/table/editor_plugin_src.js line 968  but handling div form insteadof table */
		function(ed, e)
		{
			var rng, table, dom = ed.dom;
	
			// On gecko it's not possible to place the caret before a table
			if (e.keyCode == 37 || e.keyCode == 38) 
			{
				rng = ed.selection.getRng();
				table = dom.getParent(rng.startContainer, 'table');
	
				if (table && ed.getBody().firstChild != table) // DOING THE REVERSE CONDITION BECAUSE WE WANT TO DO SO ALL THE TIME 
				{
					if (org.ametys.forms._isAtStart(rng, table)) 
					{
						rng = dom.createRng();
	
						rng.setStartBefore(table);
						rng.setEndBefore(table);
	
						ed.selection.setRng(rng);
						
						e.preventDefault();
					}
				}
			}
		}
	);
}

/**
 * @static
 * Initiate the tiny mce. Before calling this method set the tags yoo handle through the getTag method
 * @param cssFiles An optionnal list of css files to load as a comma-separated string. eg: "/plugins/cms/resources/css/mycss.css;/plugins/cms/resources/css/mycss2.css". This parameter is used only at the first call of this method
 */
org.ametys.cms.editor.Editor.init = function()
{
	if (org.ametys.cms.editor.Editor._tinyMCEInitiated)
	{
		return;
	}
	
	var cssFilesToLoad = "";
	for (var i = 0; i < org.ametys.cms.editor.Editor.useCSSFile.length; i++)
	{
		if (cssFilesToLoad.length != 0) cssFilesToLoad += ","
		cssFilesToLoad += org.ametys.cms.editor.Editor.useCSSFile[i];
	}
	
	var validElements = "";
	for (var key in org.ametys.cms.editor.Editor._tags)
	{
		if (validElements != "")
		{
			validElements += ",";
		}
		validElements += org.ametys.cms.editor.Editor._tags[key].toString();
	}

	tinyMCE.baseURL = context.contextPath + '/plugins/tiny_mce/resources/js';
	tinyMCE.init
	({
		// General options
		mode : "none",
		theme : "advanced",
		plugins : "table,paste,tabfocus,noneditable,autolink",
		doctype: "<!DOCTYPE html PUBLIC \"-//W3C//DTD XHTML 1.0 Strict//EN\" \"http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd\">",

		entity_encoding : 'raw',
		fix_list_elements : true,
		fix_table_elements : true,
		fix_nesting : true,
		verify_css_classes : false,
		gecko_spellcheck : true,
		valid_elements : validElements,
		default_valid_elements : validElements,
		paste_strip_class_attributes: "mso",
		paste_remove_styles: true,
		relative_urls : false,
		remove_script_host: false,
		document_base_url: context.contextPath + "/", 
		strict_loading_mode : true,
		language: context.parameters.userLocale,

		// Theme options
		theme_advanced_buttons1 : "",
		theme_advanced_buttons2 : "",
		theme_advanced_buttons3 : "",
	
		// Theme options
		theme_advanced_toolbar_location : "none",
		theme_advanced_path : false,
		theme_advanced_statusbar_location : "bottom",
		theme_advanced_resizing : true,
		theme_advanced_resizing_use_cookie: false,
		
		
		content_css : cssFilesToLoad,
		
		init_instance_callback: function(ed)
		{
			// Char counter
			var textarea = document.getElementById(ed.editorId);
			var maxLength = textarea.getAttribute("maxlength");
		
			Ext.get(tinymce.DOM.get(ed.id + '_path_row')).createChild({
					tag: 'span',
					style: 'padding-left: 5px; padding-right: 5px; border-right: 1px solid #222222; color: #222222; font-family: Tahoma; font-size: 0.8em;',
					html: ed.getLang('charcount_chars', 'Characters:') + ' '
						 +    '<span id="' + ed.id + '-counter-val' + '">?</span>'
						 +    (maxLength == null ? '' : (' ' +  ed.getLang('charcount_max', 'on') + ' ' + maxLength)) 
				}
			);
			
			if (textarea.getAttribute("fixedwidth") == "false")
			{
				Ext.get(tinymce.DOM.get(ed.id + '_path_row')).next("a").insertSibling({
					tag: 'img',
					title: ed.getLang('width_unknown', 'The editor width is unknown. Please set it manually'),
					src: context.contextPath +  '/plugins/cms/resources/img/content/warning.png',
					style: 'float: right; margin-top: 2px;'
				},
				'after'
			);
			}
			
			org.ametys.cms.editor.Editor._onChangeStartCount(ed);
		},

		setup: function(ed)
		{
			if (Ext.isIE)
			{
				ed.onEvent.add(tinyMCE.save);
			}
			
			ed.CHECK_EVERY = 1000;
			ed.onChange.add (org.ametys.cms.editor.Editor._onChangeStartCount);
			ed.onSetContent.add (org.ametys.cms.editor.Editor._onChangeStartCount);
			ed.onKeyUp.add (org.ametys.cms.editor.Editor._onChangeStartCount);
			
			ed.onNodeChange.add (org.ametys.cms.editor.Editor._onRichTextNodeSelected);

			for (var i = 0; i < org.ametys.cms.editor.Editor.onSetContent.length; i++)
			{
				ed.onBeforeSetContent.add(org.ametys.cms.editor.Editor.onSetContent[i]);
			}
			for (var i = 0; i < org.ametys.cms.editor.Editor.onGetContent.length; i++)
			{
				ed.onGetContent.add(org.ametys.cms.editor.Editor.onGetContent[i]);
			}
			for (var i = 0; i < org.ametys.cms.editor.Editor.onKeyPress.length; i++)
			{
				ed.onKeyPress.add(org.ametys.cms.editor.Editor.onKeyPress[i]);
			}
			for (var i = 0; i < org.ametys.cms.editor.Editor.onKeyDown.length; i++)
			{
				ed.onKeyDown.add(org.ametys.cms.editor.Editor.onKeyDown[i]);
			}
			for (var i = 0; i < org.ametys.cms.editor.Editor.onKeyUp.length; i++)
			{
				ed.onKeyUp.add(org.ametys.cms.editor.Editor.onKeyUp[i]);
			}
			for (var i = 0; i < org.ametys.cms.editor.Editor.onVisualAid.length; i++)
			{
				ed.onVisualAid.add(org.ametys.cms.editor.Editor.onVisualAid[i]);
			}
			for (var i = 0; i < org.ametys.cms.editor.Editor.onPreProcess.length; i++)
			{
				ed.onPreProcess.add(org.ametys.cms.editor.Editor.onPreProcess[i]);
			}
		}
	});
	tinyMCE.save = function()
	{
		if (Ext.isIE)
		{
			tinyMCE.activeEditor._bookmark = tinyMCE.activeEditor.selection.getBookmark(1);
		}
	}
	tinyMCE.focus = function ()
	{
		tinyMCE.activeEditor.focus();
		if (tinyMCE.activeEditor._bookmark)
		{
			tinyMCE.activeEditor.selection.moveToBookmark(tinyMCE.activeEditor._bookmark);
		}

	}
	org.ametys.cms.editor.Editor._tinyMCEInitiated = true;
}

org.ametys.cms.editor.Editor._onChangeStartCount = function(editor)
{
	editor = editor || tinyMCE.activeEditor; 
	if (editor != null)
	{
		var counter = document.getElementById(editor.editorId + "-counter-val"); 
		if (counter != null)
		{
			var time = new Date().getTime();
			var lastTime = editor._lastCountUpdate;
			editor._lastCountUpdate = time;

			if (lastTime == null || (time - lastTime >= editor.CHECK_EVERY && editor._counting != null))
			{
				org.ametys.cms.editor.Editor._SetCount(editor);
			}
			else
			{
				counter.style.color = "#7f7f7f";
				window.clearTimeout(editor._counting);
				editor._counting = window.setTimeout("org.ametys.cms.editor.Editor._SetCount()", editor.CHECK_EVERY);
			}
		}
	}
}

/**
 * @private
 * Count the characters on change
 */
org.ametys.cms.editor.Editor._SetCount = function(editor)
{
	editor = editor || tinyMCE.activeEditor; 
	if (editor != null)
	{
		var took = new Date().getTime();
		
		var r = new RegExp("<p( [^>]+)?>" + String.fromCharCode(0xA0) + "<\/p>", "g");
		var a = editor.getContent({from: 'char-counter'}).replace(/\r?\n/g, '').replace(r, '').replace(/<[^>]*>/g, '').length;
		var counter = document.getElementById(editor.editorId + "-counter-val"); 
		if (counter != null)
		{
			counter.style.color = "";
			editor._counting = null;
			counter.innerHTML = a;
		}
		
		// is there a maxlength ?
		var textarea = document.getElementById(editor.editorId);
		var maxlength = textarea.getAttribute("maxlength");
		if (maxlength != null)
		{
			if (a > parseInt(maxlength))
			{
				counter.style.color = "#ff0000";
			}
			else
			{
				counter.style.color = "";
			}
		}

		var took2 = new Date().getTime();
		editor.CHECK_EVERY = Math.max(took2 - took, 1000);
	}
}

/**
 * Suspend, when a node is selected, its transmission to the ribbon.
 * Use it when you know that several events could be sent at once : suspend before the first, and restart before the last.
 * Do not forget to call the restart method! (each suspend should have a restart - think about try/catch to ensure this)
 */
org.ametys.cms.editor.Editor.suspendRichTextNodeSelectionEvent = function()
{
	org.ametys.cms.editor.Editor._suspended++;
}
/**
 * Restart the transmission of selection event to the ribbon.
 * Do not call this method if you do not have call the suspend one before.
 */
org.ametys.cms.editor.Editor.restartRichTextNodeSelectionEvent = function()
{
	if (org.ametys.cms.editor.Editor._suspended == 0)
	{
		throw "org.ametys.cms.editor.Editor.restartRichTextNodeSelectionEvent method has been called but transmissions where not suspended";
	}

	org.ametys.cms.editor.Editor._suspended--;
}

/**
 * @private
 * When a node is selected in the rich text
 */
org.ametys.cms.editor.Editor._onRichTextNodeSelected = function(editor, controlManager, node, isCollapsed)
{
	if (tinymce.activeEditor.contentDocument == node || org.ametys.cms.editor.Editor._suspended > 0)
	{
		return;
	}
	
	if (editor.notFirstCallToOnRichTextNodeSelected)
	{
		var cmp = Ext.getCmp(editor.id); 
		var contentTool = cmp.contentTool;
		contentTool.onHTMLNodeSelected.defer(1, contentTool, [cmp.name, node]);
	}
	else
	{
		editor.onClick.remove(editor.nodeChanged);
	}
	editor.notFirstCallToOnRichTextNodeSelected = true;
}

/**
 * Insert the html given at the root of the current selection of the current editor
 * @param {String} html The html to insert
 * @param {String} mode The mode 'split' : to split the current place of the cursor to insert the html ; 'after' to inster after the root element of the current selection ; 'before' idem but before. 'split' is the default value
 */
org.ametys.cms.editor.Editor.insertHTMLAtRoot = function(html, mode)
{
	mode = mode || 'split';
	
	var rootelements = 'h1,h2,h3,h4,h5,h6,div,p';
	
	var root = tinyMCE.activeEditor.dom.getParent(tinyMCE.activeEditor.selection.getNode(), rootelements);

	if (mode == 'split' || root == null)
	{
		// UPGRADE tinyMCE 3.3.3 plugins/table/js/table.js insertTable
		tinyMCE.activeEditor.selection.setContent('<br class="_mce_marker" />');
	
		var patt = '';
		tinymce.each(rootelements.split(','), function(n) {
			if (patt)
				patt += ',';
	
			patt += n + ' ._mce_marker';
		});
	
		tinymce.each(tinyMCE.activeEditor.dom.select(patt), function(n) {
			tinyMCE.activeEditor.dom.split(tinyMCE.activeEditor.dom.getParent(n, rootelements), n);
		});
	
		tinyMCE.activeEditor.dom.setOuterHTML(tinyMCE.activeEditor.dom.select('br._mce_marker')[0], html);
	}
	else
	{
		var br = tinyMCE.activeEditor.dom.doc.createElement("br")
		root.parentNode.insertBefore(br, mode == 'after' ? root.nextSibling : root);
		tinyMCE.activeEditor.dom.setOuterHTML(br, html);
	}
}
