/*
 *  Copyright 2010 Anyware Services
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 */

Ext.namespace('org.ametys.cms.ribbon.tab');

/**
 * Do not call this contructor
 * @constructor
 * @class This class handles the tab that shows when selection is of the configured type
 * @param {String} id The unique tab identifier
 * @param {Map<id, value>} configuration The configuration of the control instance. The key 'target-type' represents the name of the target that will make the tab show when this kind of target is in the {@see org.ametys.ribbon.RibbonManager.EVENTTYPE_SELECTIONCHANGED} message. You can optionnaly set a subtarget-type that do the same on the immediate first child
 * @param {String} The name of the plugin where the control was declared
 * @param {Ext.Panel} The panel of the tab
 */
org.ametys.cms.ribbon.tab.SelectionTab = function(id, configuration, pluginName, tabPanel) 
{
	org.ametys.cms.ribbon.tab.SelectionTab.superclass.constructor.call(this, id, configuration, pluginName, tabPanel);
	
	this._targetRegExp = new RegExp(configuration['target-type']);
	if (configuration['subtarget-type'] != null)
	{
		var regExp = configuration['subtarget-type'];
		if (regExp.indexOf("!") == 0)
		{
			regExp = regExp.substring(1);
			this._subtargetRegExpReverse = true;
		}
		else
		{
			this._subtargetRegExpReverse = false;
		}
		this._subtargetRegExp = new RegExp(regExp);
	}
}

Ext.extend(org.ametys.cms.ribbon.tab.SelectionTab, org.ametys.ribbon.control.RibbonTab, {});

/**
 * @private
 * @property {RegExp} _targetRegExp The regexp for the target lookup
 */
org.ametys.cms.ribbon.tab.SelectionTab.prototype._targetRegExp;

/**
 * @private
 * @property {RegExp} _subtargetRegExp The regexp for the sub target lookup
 */
org.ametys.cms.ribbon.tab.SelectionTab.prototype._subtargetRegExp;

/**
 * Activate or deactivate the tab depending on the selection
 * @param {org.ametys.messagebus.message.Message[]} messages The messages received.
 * @private
 */
org.ametys.cms.ribbon.tab.SelectionTab.prototype.onMessage = function (messages)
{
	for (var i=0; i < messages.length; i++)
	{
		if (messages[i].getType().getName() == org.ametys.ribbon.RibbonManager.EVENTTYPE_SELECTIONCHANGED)
		{
			var targets = messages[i].getTargets();
			var target = org.ametys.messagebus.message.MessageTargetHelper.findFirst(targets, this._testTarget.createDelegate(this));

			if (target == null)
			{
				this.hide();
			}
			else
			{
				this.show(messages[i].getType().getParameters() != null && messages[i].getType().getParameters()["creation"] == target.getType());
			}
		}
	}
	
}

/**
 * @private
 * Test a target. Does it match ?
 * @return {org.ametys.messagebus.message.MessageTarget} The target to check
 */
org.ametys.cms.ribbon.tab.SelectionTab.prototype._testTarget = function(target)
{
	return this._targetRegExp.test(target.getType()) 
		&& (this._subtargetRegExp == null 
				|| (!this._subtargetRegExpReverse && target.getSubTargets().length > 0 && this._subtargetRegExp.test(target.getSubTargets()[0].getType()))
				|| (this._subtargetRegExpReverse && (target.getSubTargets().length == 0 || !this._subtargetRegExp.test(target.getSubTargets()[0].getType())))
	); 
}
