/*
 *  Copyright 2010 Anyware Services
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 */

// Ametys Namespace
Ext.namespace('org.ametys.fluent');

/**
 * Creates a ribbon group in a ribbon tab
 * 
 * @class This class is the panel for building group of controls in a ribbon
 * @extends Ext.Panel
 * @constructor
 * @param {Object} config Configuration options
 */
org.ametys.fluent.Group = function(config) 
{
	if (config.smallItems && config.smallItems.length > 0)
	{
		this.smallContainer = new Ext.Container({
			items: config.smallItems, 
			hidden: true,
			layout: 'table'
		});
	}

	this.mediumContainer = new Ext.Container({
		items: config.items, 
		hidden: true,
		layout: 'table'
	});
	
	if (config.largeItems && config.largeItems.length > 0)
	{
		this.largeContainer = new Ext.Container({
			items: config.largeItems, 
			hidden: true,
			layout: 'table'
		});
	}
	
	if (this.largeContainer)
	{
		this.scale = "large";
		this.largeContainer.show();
	}
	else if (this.mediumContainer)
	{
		this.scale = "medium";
		this.mediumContainer.show();
	}
	else if (this.smallContainer)
	{
		this.scale = "small";
		this.smallContainer.show();
	}
	else
	{
		throw "Group has no items in any scale";
	}
		
	
	config.items = [];
	if (this.smallContainer)
	{
		config.items.push(this.smallContainer);
	}
	config.items.push(this.mediumContainer);
	if (this.largeContainer)
	{
		config.items.push(this.largeContainer);
	}
	
	org.ametys.fluent.Group.superclass.constructor.call(this, config);

	this.on('render', this._onRenderModify, this);
	if (Ext.isIE7)
	{
		this.on('afterlayout', this._onLayoutForIE7, this);
	}
	if (Ext.isIE6)
	{
		this.on('afterlayout', this._onLayoutForIE6, this);
	}

    this.addEvents
    (
            /**
             * @event dialogboxlaunch
             * Fires when the dialog box button was clicked
             * @param {Ext.Panel} p the current Panel.
             */
            'dialogboxlaunch'
    );
}

Ext.extend
(
	org.ametys.fluent.Group, 
	Ext.Panel, 
	{
		layout: 'table',
		cls: 'x-fluent-group',
		frame: true,
		forceLayout: true
	}
);

org.ametys.fluent.Group.prototype.getScale = function()
{
	return this.scale;
}

org.ametys.fluent.Group.prototype.supportScale = function(newSize)
{
	return this[newSize + "Container"] != null;
}

org.ametys.fluent.Group.prototype.setScale = function(newSize)
{
	if (newSize == this.scale || !this.supportScale(newSize))
	{
		return;
	}
	
	this[this.scale + "Container"].hide();
	this.scale = newSize;
	this[this.scale + "Container"].show();
}

org.ametys.fluent.Group.prototype.canBeVisible = function()
{
	return this.el ? this.el.getTop() < 60 && this.el.getRight() < this.el.parent().getRight() : true;
}

/**
 * private
 * Listener on layout to adapt title width
 */
org.ametys.fluent.Group.prototype._onLayoutForIE7 = function()
{
    this.header.setWidth("0px");

    var bodyWidth = this.body.getFrameWidth('lr') + this.body.dom.firstChild.offsetWidth;
    this.header.setWidth(bodyWidth);
}

/**
 * private
 * Listener on layout to adapt title width
 */
org.ametys.fluent.Group.prototype._onLayoutForIE6 = function()
{
	this.body.setWidth("auto");
    this.el.setWidth("auto");
    this.header.setWidth("auto");

    var bodyWidth = this.body.getFrameWidth('lr') + this.body.dom.firstChild.offsetWidth;
    this.header.setWidth(bodyWidth);
    this.body.setWidth(bodyWidth);
    this.el.setWidth(bodyWidth + this.getFrameWidth());
}

/**
 * @private
 * Listener on render to modify the order of header, add the fade listeners and add the dialogbox launcher
 * @param {Ext.Component} component The main component
 */
org.ametys.fluent.Group.prototype._onRenderModify = function(component)
{
	// move header at the last position
	this.el.dom.appendChild(this.el.dom.firstChild);
	
	this.el.hover(this._onOverDoFadeIn, this._onOutDoFadeOut, this);
	
	// add dialog box launcher
	if (this.dialogBoxLauncher)
	{
		var a = this.header.insertFirst({cls: 'x-fluent-dialogboxlauncher'});
		a.addClassOnOver('x-fluent-dialogboxlauncher-over');
		a.addClassOnClick('x-fluent-dialogboxlauncher-active');
		a.on('click', this._onDialogBoxLauncherClickSendEvent, this);
	}
}

/**
 * @private
 * Listener on dialog box launcher click to launch the event
 */
org.ametys.fluent.Group.prototype._onDialogBoxLauncherClickSendEvent = function ()
{
	this.fireEvent('dialogboxlaunch', this);
}

/**
 * @private
 * Listener on over to do fade in 
 */
org.ametys.fluent.Group.prototype._onOverDoFadeIn = function()
{
	var nbFrames = 8 - 1;
	var frameSize = 264;
	
	this.el.child("div.x-panel-tl").anim({bgscroll: {"to": [0, - 70 - frameSize * nbFrames], "by": [1, frameSize]}}, null, 'bgScroll', 0.5, "easeNone");
	this.el.child("div.x-panel-tc").anim({bgscroll: {"to": [0, - 246 - frameSize * nbFrames], "by": [1, frameSize]}}, null, 'bgScroll', 0.5, "easeNone");
	this.el.child("div.x-panel-tr").anim({bgscroll: {"to": [0, - 158 - frameSize * nbFrames], "by": ["right", frameSize]}}, null, 'bgScroll', 0.5, "easeNone");
	this.el.child("div.x-panel-ml").anim({bgscroll: {"to": [0, - 0 - frameSize * nbFrames], "by": [1, frameSize]}}, null, 'bgScroll', 0.5, "easeNone");
	this.el.child("div.x-panel-mc").anim({bgscroll: {"to": [0, - 176 - frameSize * nbFrames], "by": [1, frameSize]}}, null, 'bgScroll', 0.5, "easeNone");
	this.el.child("div.x-panel-mr").anim({bgscroll: {"to": [0, - 88 - frameSize * nbFrames], "by": ["right", frameSize]}}, null, 'bgScroll', 0.5, "easeNone");
}

/**
 * @private
 * Listener on over to do fade out 
 */
org.ametys.fluent.Group.prototype._onOutDoFadeOut = function()
{
	var nbFrames = 8 - 1;
	var frameSize = 264;
	
	this.el.child("div.x-panel-tl").anim({bgscroll: {"to": [0, - 70], "by": [1, frameSize]}}, null, 'bgScroll', 0.5, "easeNone");
	this.el.child("div.x-panel-tc").anim({bgscroll: {"to": [0, - 246], "by": [1, frameSize]}}, null, 'bgScroll', 0.5, "easeNone");
	this.el.child("div.x-panel-tr").anim({bgscroll: {"to": [0, - 158], "by": ["right", frameSize]}}, null, 'bgScroll', 0.5, "easeNone");
	this.el.child("div.x-panel-ml").anim({bgscroll: {"to": [0, - 0], "by": [1, frameSize]}}, null, 'bgScroll', 0.5, "easeNone");
	this.el.child("div.x-panel-mc").anim({bgscroll: {"to": [0, - 176], "by": [1, frameSize]}}, null, 'bgScroll', 0.5, "easeNone");
	this.el.child("div.x-panel-mr").anim({bgscroll: {"to": [0, - 88], "by": ["right", frameSize]}}, null, 'bgScroll', 0.5, "easeNone");
}

org.ametys.fluent.Group.Part = function(config)
{
	if (config.align == 'middle')
	{
		config.cls = 'x-fluent-group-part-middle';
	}
	else
	{
		config.cls = 'x-fluent-group-part-top';
	}
	
	org.ametys.fluent.Group.Part.superclass.constructor.call(this, config);
}

Ext.extend
(
	org.ametys.fluent.Group.Part, 
	Ext.Panel, 
	{
		layout: 'table',
		cls: 'x-fluent-group-part'
	}
);
