/*
 *  Copyright 2010 Anyware Services
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 */

// Ametys Namespace
Ext.namespace('org.ametys.fluent');

/**
 * Creates a ribbon panel. Sort of tab panel with toolbars in it.
 * @class This class is the main panel for building a ribbon
 * @extends Ext.TabPanel
 * @constructor
 * @param {Object} config Configuration options
 */
org.ametys.fluent.RibbonPanel = function(config) 
{
	this._ribbonCollapsed = false;
	this._ribbonFloating = false;
	this._lastContextualTabHidden = null;
	this._actionPerfomed = false;
	this._contextualGroups = {};
	this._scaleGrid = {};

	config.itemTpl = new Ext.Template
	(
		'<li class="{cls}" id="{id}"><div><a class="x-tab-strip-close" onclick="return false;"></a>',
		'<a class="x-tab-right" href="#" onclick="return false;"><em class="x-tab-left">',
		'<span class="x-tab-strip-inner"><span class="x-tab-strip-text {iconCls}">{text}</span></span>',
		'</em></a></div></li>'
	)


	org.ametys.fluent.RibbonPanel.superclass.constructor.call(this, config);

	this.on('render', this._onRender, this);

	this.appButton = new Ext.Button
	( 
		{
			cls :'x-fluent-appbutton' + (config.menu.items ? '' : ' x-fluent-appbutton-empty-menu'),
			tooltip :config.menu ? config.menu.tooltip : null,
			menu :config.menu && config.menu.items ? new Ext.menu.Menu({ items :config.menu.items }) : null,
			handler :config.menu && config.menu.handler
		}
	);
}

/**
 * @constant
 * @type {integer}
 * @private The height of the header part of the ribbon
 */
org.ametys.fluent.RibbonPanel.HEADER_HEIGHT = 51;
/**
 * @constant
 * @type {integer}
 * @private The height of the body part of the ribbon
 */
org.ametys.fluent.RibbonPanel.BODY_HEIGHT = 92;

/**
 * @config
 * @property {String} applicationTitle The prefix in the ribbon title bar
 */
Ext.extend
(
	org.ametys.fluent.RibbonPanel, 
	Ext.TabPanel, 
	{
		border: false,
		resizeTabs: false,
		deferredRender: true,
	
		activeTab: 0,
		cls: 'x-fluent-tab-panel',
		height: org.ametys.fluent.RibbonPanel.HEADER_HEIGHT + org.ametys.fluent.RibbonPanel.BODY_HEIGHT
	}
);

/**
 * @private
 * @property {boolean} _ribbonCollapsed Is the ribbon currently in collapsed mode
 */
org.ametys.fluent.RibbonPanel.prototype._ribbonCollapsed;
/**
 * @private
 * @property {Ext.Panel} _lastContextualTabHidden Last contextual panel selected. Null when hidden for too long.
 */
org.ametys.fluent.RibbonPanel.prototype._lastContextualTabHidden;
/**
 * @private
 * @property {Map<String, Ext.Panel>} _contextualGroups The map of contextual group id and the associated label panel
 */
org.ametys.fluent.RibbonPanel.prototype._contextualGroups;
/**
 * @private
 * @property {boolean} _actionPerfomed Number of click performed since the last contextual tab hidden
 */
org.ametys.fluent.RibbonPanel.prototype._actionPerfomed;
/**
 * @private
 * @property {boolean} _ribbonFloating Is the ribbon currently in collapsed mode but visible
 */
org.ametys.fluent.RibbonPanel.prototype._ribbonFloating;

/**
 * @private Listener when rendering the tab
 * @param {Ext.Container} ct The container of the panel
 */
org.ametys.fluent.RibbonPanel.prototype._onRender = function(component) 
{
	this.header.titleWrapper = this.header.insertFirst({ cls: 'x-tab-panel-header-text-wrapper' });
	this.header.title = this.header.titleWrapper.insertFirst({ cls: 'x-tab-panel-header-text' });
	
	this.setTitle(this.title);

	this.header.button = this.header.insertFirst({ cls: 'x-fluent-appbutton-wrapper' });
	this.appButton.render(this.header.button);

	this.getEl().dom.style.zIndex = 1;

	this.on('beforetabchange', this._onBeforeTabChanged, this);
	this.on('tabchange', this._onTabChanged, this);
	this.strip.on('dblclick', this.toggleRibbonCollapse, this);
	this.getEl().on('mousewheel', this._onTabWheel, this);

	Ext.getDoc().on("click", this._onAnyMouseDown, this);
	Ext.getDoc().on("contextmenu", this._onAnyMouseDown, this);
	this.on('afterlayout', this._onLayoutChanged, this);
	this.on('resize', this._onResize, this);
	
	if (this.menu && this.menu.icon) 
	{
		this.getEl().child(".x-tab-panel-header .x-fluent-appbutton-wrapper button").dom.style.backgroundImage = "url('" + this.menu.icon + "')";
	}

	this.items.each(this._initiateTab, this);
}

/**
 * @private
 * Listener when tab has changer to ensure a correct lyout
 */
org.ametys.fluent.RibbonPanel.prototype._onTabChanged = function()
{
	this.activeTab.doLayout();
	
	this._checkSize();
}
/**
 * Set the title in the ribbon. The title is prefixed by the application title {@see org.ametys.fluent.RibbonPanel.applicationTitle}
 * @param {String} title The title to set in the Ribbon. Can be null or empty.
 */
org.ametys.fluent.RibbonPanel.prototype.setTitle = function(title) 
{
	this.title = title;

	var fullTitle = (this.title != null && this.title.length != 0 ? this.title + " - " : "")
			+ (this.applicationTitle != null && this.applicationTitle.length != 0 ? ("<span>" + this.applicationTitle + "</span>") : "");

	this.header.title.update(fullTitle);
}

/**
 * Toggle the ribbon state
 */
org.ametys.fluent.RibbonPanel.prototype.toggleRibbonCollapse = function() 
{
	this[this._ribbonCollapsed ? 'expandRibbon' : 'collapseRibbon']();
	return this;
}

/**
 * Enter in collapsed mode
 * @param {boolean} animate If animation is required
 */
org.ametys.fluent.RibbonPanel.prototype.collapseRibbon = function() 
{
	this._ribbonCollapsed = true;
	this._ribbonFloating = true;

	this._hideCollapsed();

	return this;
}
/**
 * @private 
 * When in collapsed mode but visible : hide it
 */
org.ametys.fluent.RibbonPanel.prototype._hideCollapsed = function() 
{
	if (this._ribbonCollapsed && this._ribbonFloating) 
	{
		this.body.setDisplayed(false);
		this._ribbonFloating = false;

		this.ownerCt.doLayout();

		// Deactivate the active tab
		if (this.activeTab) 
		{
			var oldEl = this.getTabEl(this.activeTab);
			if (oldEl) 
			{
				Ext.fly(oldEl).removeClass('x-tab-strip-active');
			}
			this.activeTab.fireEvent('deactivate', this.activeTab);
			this.activeTab = null;
		}
	}
}

/**
 * Expand the ribbon
 */
org.ametys.fluent.RibbonPanel.prototype.expandRibbon = function() 
{
	this._showCollapsed();
	this._ribbonCollapsed = false;

	this.ownerCt.doLayout();

	return this;
}

/**
 * @private When in collapsed mode, show it
 */
org.ametys.fluent.RibbonPanel.prototype._showCollapsed = function() 
{
	if (this._ribbonCollapsed && !this._ribbonFloating) 
	{
		this.body.setDisplayed(true);
		this._ribbonFloating = true;

		var el = this.getTabEl(this.lastActiveTab);
		if (el.style.display != 'none')
		{
			this.setActiveTab(this.lastActiveTab);
		}
		else
		{
			this.setActiveTab(0);
		}
	}
}

/**
 * @private Listener when layout is made up
 */
org.ametys.fluent.RibbonPanel.prototype._onResize = function() 
{
	this._checkSize();
}

org.ametys.fluent.RibbonPanel.prototype._buildScalesGrid = function(index)
{
	var scaleGrid = this._scaleGrid; 
	scaleGrid[index] = [];

	var tab = this.items.get(index);
	
	// compute the priority between groups
	var priority = [];
	tab.items.each(function (e, i)
	{
		priority.push({index: i, priority: e.priority ? e.priority : 0});
	});
	function compare(e1, e2)
	{
		return (e1.priority != e2.priority) ? e1.priority < e2.priority : e2.index < e1.index;
	}
	priority.sort(compare);
	
	// fill the matrix
	function fill(first, second, fullspeed)
	{
		for (var cursor = fullspeed ? 1 : 0; cursor <= priority.length; cursor++)
		{
			var line = [];
			for (var i = 0; i < priority.length; i++) { line.push("X"); }

			for (var i = 0; i < priority.length - cursor; i++)
			{
				line[priority[i].index] = first;
			}
			for (var i = 0; i < cursor; i++)
			{
				line[priority[priority.length - cursor + i].index] =second;
			}
			
			scaleGrid[index].matrix.push({resize: line});
		}
	}
	
	scaleGrid[index].matrix = [];
	scaleGrid[index].current = 0;

	fill("large", "medium", false);
	fill("medium", "small", true);
	
	// change unexisting combo & remove duplicates
	for (var i = scaleGrid[index].matrix.length - 1; i >= 0 ; i--)
	{
		var grid = scaleGrid[index].matrix[i];
		for (var j = 0; j < grid.resize.length; j++)
		{
			if (!tab.items.get(j).supportScale(grid.resize[j]))
			{
				grid.resize[j] = "medium";
			}
		}
		
		if (i + 1 < scaleGrid[index].matrix.length)
		{
			var areTheSame = true;
			
			var grid = scaleGrid[index].matrix[i];
			var grid2 = scaleGrid[index].matrix[i + 1];
			for (var j = 0; j < grid.resize.length; j++)
			{
				if (grid.resize[j] != grid2.resize[j])
				{
					areTheSame = false;
					break;
				}
			}	
			
			if (areTheSame)
			{
				scaleGrid[index].matrix.remove(grid2);
			}
		}
	}
}
		
org.ametys.fluent.RibbonPanel.prototype._checkSize = function()
{
	if (this.getActiveTab())
	{
		var doesNotFit = false;
		this.getActiveTab().items.each(function (e)
		{
			if (e.canBeVisible && !e.canBeVisible())
			{
				doesNotFit = true;
			}
		});
		
		var index = this.items.indexOf(this.getActiveTab());
		if (this._scaleGrid[index] == null)
		{
			this._buildScalesGrid(index);
		}
		
		var grid = this._scaleGrid[index];

		if (doesNotFit)
		{
			if (!grid.matrix[grid.current].doesNotFitIn || grid.matrix[grid.current].doesNotFitIn < this.getActiveTab().getWidth())
			{
				grid.matrix[grid.current].doesNotFitIn = this.getActiveTab().getWidth();
			}

			if (grid.current + 1 < grid.matrix.length)
			{
				grid.current++;
				
				for (var i = 0; i < grid.matrix[grid.current].resize.length; i++)
				{
					this.getActiveTab().items.get(i).setScale(grid.matrix[grid.current].resize[i]);
				}
				
				// may need to go under !
				this._checkSize();
			}
			else
			{
				// does not fit in the window, but already at the min size
				return;
			}
		}
		else
		{
			if (!grid.matrix[grid.current].fitIn || grid.matrix[grid.current].fitIn > this.getActiveTab().getWidth())
			{
				grid.matrix[grid.current].fitIn = this.getActiveTab().getWidth();
			}
			
			// Can we grow ?
			if (grid.current > 0)
			{
				if (!grid.matrix[grid.current - 1].doesNotFitIn || this.getActiveTab().getWidth() > grid.matrix[grid.current - 1].doesNotFitIn)
				{
					// have a try
					grid.current--;
					
					for (var i = 0; i < grid.matrix[grid.current].resize.length; i++)
					{
						this.getActiveTab().items.get(i).setScale(grid.matrix[grid.current].resize[i]);
					}

					this._checkSize();
				}
			}
		}
	}
}

/**
 * @private Listener when layout is made up
 */
org.ametys.fluent.RibbonPanel.prototype._onLayoutChanged = function() 
{
	this.body.setHeight(org.ametys.fluent.RibbonPanel.BODY_HEIGHT);
}

/**
 * @private Listener when mouse wheel is used over tabs
 * @param {Event} e The event
 */
org.ametys.fluent.RibbonPanel.prototype._onTabWheel = function(e) 
{
	if (this._ribbonCollapsed) 
	{
		return;
	}

	var scrollSens = (e.getWheelDelta() > 0)
	var initialValue = scrollSens ? 1 : 0;
	var increment = scrollSens ? -1 : 1;

	for ( var i = initialValue; i < this.items.length + 1 - initialValue; i++) 
	{
		if (this.items.get(i) == this.getActiveTab()) 
		{
			for ( var j = i + increment; j < this.items.length + 1 - initialValue; j += increment)
			{
				if (Ext.get(this.getTabEl(j)).isVisible())
				{
					this.setActiveTab(this.items.get(j));
					e.stopEvent();
					Ext.menu.MenuMgr.hideAll();
					return;
				}
			}
			return;
		}
	}
}

/**
 * @private 
 * Listener that remember last active tab and display on collapse mode
 * @param {Ext.TabPanel} tabpanel The tab panel
 * @panel {Ext.Panel} panel The panel
 */
org.ametys.fluent.RibbonPanel.prototype._onBeforeTabChanged = function(tabpanel, panel) 
{
	this.lastActiveTab = panel;

	var activeTab = this.getActiveTab();
	if (activeTab) 
	{
		// Effect deactivated on Opera9, IE6 and IE7 for performance purposes
		if (!Ext.isOpera && !Ext.isIE6 && !Ext.isIE7 && activeTab.contextualTab != null)
		{
			this.getEl().removeClass("x-fluent-tab-panel-contextual-" + activeTab.contextualTab);
		}
	}
	
	// Effect deactivated on Opera9, IE6 and IE7 for performance purposes
	if (!Ext.isOpera && !Ext.isIE6 && !Ext.isIE7 && panel != null && panel.contextualTab != null)
	{
		this.getEl().addClass("x-fluent-tab-panel-contextual-" + panel.contextualTab);
	}

	if (this._ribbonCollapsed) 
	{
		this._showCollapsed();
	}
}

/**
 * @private 
 * Listener on any click. If out of the ribbon collapse it (on collapsed mode)
 * @param {Event} e The event
 */
org.ametys.fluent.RibbonPanel.prototype._onAnyMouseDown = function(e) 
{
	var target = e.target;
	var liTagOnMyWay = false;

	do 
	{
		if (target.tagName && target.tagName.toLowerCase() === 'li') 
		{
			liTagOnMyWay = true;
		}

		if (target == this.body.dom || (target == this.header.dom && (liTagOnMyWay || e.button != 0))) 
		{
			if (e.button != 0)
			{
				e.preventDefault();
			}
			
			return;
		}
		target = target.parentNode;
	} 
	while (target != null && target.tagName != null && target.tagName.toLowerCase() != 'html');

	if (this._ribbonCollapsed && this._ribbonFloating) 
	{
		this._hideCollapsed();
	}
	
	if (this._actionPerfomed)
	{
		this._lastContextualTabHidden = null;
	}
	this._actionPerfomed = true;
}

/**
 * @private
 * Listener on adding tab to check contextual tabs
 * @param {Ext.Panel} panel The panel added
 */
org.ametys.fluent.RibbonPanel.prototype._initiateTab = function(panel)
{
	var el = Ext.get(this.getTabEl(panel));
	el.child("div").addClassOnOver("x-tab-strip-over");
	
	if (panel.contextualTab != null)
	{
		if (panel.contextualGroup == null)
		{
			panel.contextualGroup = Ext.id();
		}

		this.hideTabStripItem(panel);

		el.addClass("x-fluent-tab-contextual");
		el.addClass("x-fluent-tab-contextual-" + panel.contextualTab);
		
		if (this._contextualGroups[panel.contextualGroup] == null)
		{
			panel.ribbonLabel = this.header.titleWrapper.createChild ({ cls: "x-fluent-tab-contextuallabel-" + panel.contextualTab });
			panel.ribbonLabel.inner = panel.ribbonLabel.createChild ({ cls: "x-fluent-tab-contextuallabel-wrapper" });
			panel.ribbonLabel.inner.inner = panel.ribbonLabel.inner.createChild
			({ 
				cls: "x-fluent-tab-contextuallabel-body", 
				html: panel.contextualLabel,
				title: panel.contextualLabel
			});
			panel.ribbonLabel.setVisibilityMode(Ext.Element.DISPLAY);
			panel.ribbonLabel.hide();
			
			panel.ribbonLabel.on('click', this._onLabelClick, this, panel.ribbonLabel);
			
			this._contextualGroups[panel.contextualGroup] = panel.ribbonLabel;
			
			panel.ribbonLabel.tabs = [];
		}
		else
		{
			panel.ribbonLabel = this._contextualGroups[panel.contextualGroup];
		}
		panel.ribbonLabel.tabs.push(panel);
		
		panel.showContextualTab = function(forceSelection)
		{
			this.ownerCt.showContextualTab(this, forceSelection);
		}
		panel.hideContextualTab = function()
		{
			this.ownerCt.hideContextualTab(this);
		}
	}
	else
	{
		// checking that non contextual tabs must not be after contextual
		var cursor = el.dom;
		while (cursor.previousSibling != null && Ext.get(cursor.previousSibling).hasClass("x-fluent-tab-contextual"))
		{
			cursor = cursor.previousSibling;
		}
		
		if (cursor != el.dom)
		{
			// move
			el.dom.parentNode.insertBefore(el.dom, cursor);
		}
	}
}

/**
 * @private
 * Listener when clicking on a tab header
 * @params {Event} e The event
 * @params {Object} target The html object 
 * @params {Ext.Panel} ribbonPanel The ribbon label panel
 */
org.ametys.fluent.RibbonPanel.prototype._onLabelClick = function(e, target, ribbonPanel)
{
	this.setActiveTab(this._getVisibleContextualTabs(ribbonPanel)[0]);
}
/**
 * @private
 * Get the visible contextual tabs in the contextual group of the ribbon panel
 * @params {Ext.Panel} ribbonPanel The ribbon label panel
 * @return {Ext.Panel[]} The tab selection (not the strip panel) that are visible in the contextual group
 */
org.ametys.fluent.RibbonPanel.prototype._getVisibleContextualTabs = function(ribbonPanel)
{
	var visibles = [];
	for (var i = 0; i < ribbonPanel.tabs.length; i++)
	{
		var tab = ribbonPanel.tabs[i];
		if (this.getTabEl(tab).style.display != 'none')
		{
			visibles.push(tab);
		}
	}
	return visibles;
}
/**
 * @private
 * Set the width of the ribbonPanel to the sum of the width of the strip of the tabs of the given ribbon panel (contextual group label)
 * @params {Ext.Panel} ribbonPanel The ribbon panel (contextual tab label)
 */
org.ametys.fluent.RibbonPanel.prototype._setTotalWidthOrHide = function(ribbonPanel)
{
	var tabs = this._getVisibleContextualTabs(ribbonPanel)
	
	if (tabs.length > 0)
	{
		var sum = 0;
		for (var i = 0; i < tabs.length; i++)
		{
			var panel = tabs[i];
			var el = Ext.get(this.getTabEl(panel));
			sum += el.getWidth();
			
			el.removeClass("x-fluent-tab-contextual-" + panel.contextualTab + "-left");
			if (i == 0)
			{
				el.addClass("x-fluent-tab-contextual-" + panel.contextualTab + "-left");
			}

			el.child("div:first").removeClass("x-tab-strip-divwrapper");
			if (i == tabs.length - 1)
			{
				el.child("div:first").addClass("x-tab-strip-divwrapper");
			}
		}
	
		ribbonPanel.setWidth(sum + 2*(tabs.length - 1));
		ribbonPanel.show();
	}
	else
	{
		ribbonPanel.hide();
	}
}

/**
 * Make the contextual tab visible
 * @param {Ext.Panel} panel The tab panel to show
 * @param {boolean} forceSelection Will make the tab selected.
 */
org.ametys.fluent.RibbonPanel.prototype.showContextualTab = function(panel, forceSelection)
{
	if (panel.contextualTab != null && this.getTabEl(panel).style.display == "none")
	{
		this.unhideTabStripItem(panel);
		
		this._setTotalWidthOrHide(panel.ribbonLabel)
		
		this._computeContextualTabsLabels();
	}
	
	if (panel.activatedOnce == null || panel.wasActiveOnHide == true || forceSelection || (this._lastContextualTabHidden == panel))
	{
		panel.activatedOnce = true;
		panel.wasActiveOnHide = false;
		this.setActiveTab(panel);
	}
}

/**
 * Make the contextual tab invisible
 * @param {Ext.Panel} panel The tab panel to hide
 */
org.ametys.fluent.RibbonPanel.prototype.hideContextualTab = function(panel)
{
	this._lastContextualTabHidden = null;
	this._actionPerfomed = false;

	if (panel.contextualTab != null && this.getTabEl(panel).style.display == "")
	{
		this.hideTabStripItem(panel);

		this._setTotalWidthOrHide(panel.ribbonLabel);

		panel.wasActiveOnHide = this.activeTab == panel; 
		
		if (panel.wasActiveOnHide)
		{
			var index = panel.ribbonLabel.tabs.indexOf(panel)

			var panelToTests = [];
			for (var i = index - 1; i >= 0; i--)
			{
				var tab = panel.ribbonLabel.tabs[i];
				panelToTests.push(tab);
			}
			for (var i = index + 1; i < panel.ribbonLabel.tabs.length; i++)
			{
				var tab = panel.ribbonLabel.tabs[i];
				panelToTests.push(tab);
			}
			for (var i = this.items.length - 1; i >= 0; i--)
			{
				var tab = this.items.get(i);
				if (tab.ribbonLabel != null)
				{
					for (var j = 0; j < tab.ribbonLabel.tabs.length; j++)
					{
						var tab = tab.ribbonLabel.tabs[j];
						panelToTests.push(tab);
					}
				}
			}
			
			var done = false;
			for (var i = 0; i < panelToTests.length; i++)
			{
				var tab = panelToTests[i];
				if (this.getTabEl(tab).style.display != 'none')
				{
					this.setActiveTab(tab);
					done = true;
					break;
				}
			}
					
			if (!done)
			{
				this.setActiveTab(0);
			}
			
			this._lastContextualTabHidden = panel;
		}

		this.changeWasActiveOnHideStatus(panel);
		this._computeContextualTabsLabels();
	}	
}

/** CAN BE OVERRIDEN IN APPLICATIONS */
org.ametys.fluent.RibbonPanel.prototype.changeWasActiveOnHideStatus = function(panel)
{
	// Finally wa may want to remove this flag if activation was too recent to be true
	if (panel.wasActiveOnHide && this._lastHide != null)
	{
		var d = new Date().getTime();
		if (d - this._lastHide < 500)
		{
			panel.wasActiveOnHide = false;
		}
	}
	this._lastHide = new Date().getTime();	
}

/**
 * @private
 * Compute the header size for contextual tabs labels
 */
org.ametys.fluent.RibbonPanel.prototype._computeContextualTabsLabels = function()
{
	// look for first contextual tab
	var contextualTabStripAtLeft = null;
	for (var i = 0; i < this.items.length; i++)
	{
		var panel = this.items.get(i); 
		if (panel.contextualTab != null)
		{
			var el = this.getTabEl(panel);
			if (el.style.display != 'none')
			{
				contextualTabStripAtLeft = Ext.get(el);
				break;
			}
		}
	}
	
	if (contextualTabStripAtLeft == null)
	{
		this.header.title.setWidth("auto");
		this.header.title.setStyle("float", "none");
	}
	else
	{
		this.header.title.setWidth(10);
		this.header.title.setStyle("float", "left");

		var tabStripLeft = contextualTabStripAtLeft.getLeft();
		var titleLeft = this.header.title.getLeft();

		this.header.title.setWidth(tabStripLeft - titleLeft);
	}
}
