/*
 *  Copyright 2011 Anyware Services
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 */

Ext.namespace('org.ametys.dao.content');

/**
 * @class This class is the representation of a content. Do not keep it for a long time: informations can be obsoletes. 
 */

/**
 * Do not call this method. 
 * To get a content use the ContentDAO 
 * @constructor
 * @private
 * @param {Node} node The node sent by /plugins/cms/contents/info
 */
org.ametys.dao.content.Content = function(node)
{
	this._id = node.getAttribute("id"); 
	this._title = node.getAttribute("title"); 
	this._path = node.getAttribute("path"); 
	this._lang = node.getAttribute("lang"); 
	this._name = node.getAttribute("name"); 
	this._type = node.getAttribute("type"); 
	this._workflowName = node.getAttribute("workflowName"); 
	this._workflowSteps = org.ametys.dao.content.Content._convertStringToArray(node.getAttribute("workflowStep")); 
	this._isModifiable = (node.getAttribute("isModifiable") != null) ? ("true" == node.getAttribute("isModifiable")) : false;

	this._rights = {};
	var rightNodes = node.selectNodes ('user-rights/right');
	for (var j=0; j < rightNodes.length; j++)
	{
		this._rights[rightNodes[j].getAttribute("id")] = true;
	}
}

/**
 * @private
 * @property {String} _id The unique id of the content
 */
org.ametys.dao.content.Content.prototype._id;
/**
 * @private
 * @property {String} _exists True if the content still exists
 */
org.ametys.dao.content.Content.prototype._exists;
/**
 * @private
 * @property {String} _title The title of the content
 */
org.ametys.dao.content.Content.prototype._title;
/**
 * @private
 * @property {String} _path The path in the repository of the content
 */
org.ametys.dao.content.Content.prototype._path;
/**
 * @private
 * @property {String} _lang The lang of the content
 */
org.ametys.dao.content.Content.prototype._lang;
/**
 * @private
 * @property {String} _name The name of the content
 */
org.ametys.dao.content.Content.prototype._name;
/**
 * @private
 * @property {String} _type The content type
 */
org.ametys.dao.content.Content.prototype._type;
/**
 * @private
 * @property {String} _workflowName The identifier of the workflow used for this content
 */
org.ametys.dao.content.Content.prototype._workflowName;
/**
 * @private
 * @property {long[]} _workflowSteps The arrays of current workflow steps
 */
org.ametys.dao.content.Content.prototype._workflowSteps;
/**
 * @private
 * @property {boolean} _isModifiable True if this content can be modified. False if it read-only: this is not about rights. Can be false for a virtual content dinamically created by reading a RSS or a database... but basiccaly not stored in JCR
 */
org.ametys.dao.content.Content.prototype._isModifiable;
/**
 * @private
 * @property {Map<String,boolean>} A set of the id of the rigths the current user have on this content. The boolean is always true.
 */
org.ametys.dao.content.Content.prototype._rights;

/**
 * Get the id of the content
 * @return {String} The unique id of the content
 */
org.ametys.dao.content.Content.prototype.getId = function()
{
	return this._id;
}

/**
 * Get the title of the content
 * @return {String} The title of the content
 */
org.ametys.dao.content.Content.prototype.getTitle = function()
{
	return this._title;
}

/**
 * Get the path of the content
 * @return {String} The path of the content in the repository
 */
org.ametys.dao.content.Content.prototype.getPath = function()
{
	return this._path;
}

/**
 * Get the lang of the content
 * @return {String} The lang of the content
 */
org.ametys.dao.content.Content.prototype.getLang = function()
{
	return this._lang;
}

/**
 * Get the name of the content
 * @return {String} The name of the content
 */
org.ametys.dao.content.Content.prototype.getName = function()
{
	return this._name;
}

/**
 * Get the content type
 * @return {String} The content type
 */
org.ametys.dao.content.Content.prototype.getType = function()
{
	return this._type;
}

/**
 * Get the identifier of the workflow used for this content
 * @return {String} The identifier of the workflow used for this content
 */
org.ametys.dao.content.Content.prototype.getWorkflowName = function()
{
	return this._workflowName;
}

/**
 * Get the workflow steps
 * @return {long[]} The workflow steps
 */
org.ametys.dao.content.Content.prototype.getWorkflowSteps = function()
{
	return this._workflowSteps;
}

/**
 * Get the workflow steps
 * @param {long} A workflow step
 * @return {boolean} True if the given step is currently active
 */
org.ametys.dao.content.Content.prototype.isOnWorkflowStep = function(id)
{
	return this._workflowSteps.indexOf(id) >= 0 ;
}

/**
 * Determines whether this content is modifiable or not (this is not relative to your rights but to the intresect content)
 * @return {boolean} True if this content can be modified. False if it read-only: this is not about rights. Can be false for a virtual content dinamically created by reading a RSS or a database... but basiccaly not stored in JCR
 */
org.ametys.dao.content.Content.prototype.isModifiable = function()
{
	return this._isModifiable;
}

/**
 * Get the set of the rights the user have on this content
 * @return {Map<String,boolean>} The set of ids of rights. Do not modify it!
 */
org.ametys.dao.content.Content.prototype.getRights = function()
{
	return this._rights;
}

/**
 * Determine if the current user has the specified right on the content
 * @param {String} rightId The identifier of the right to check
 * @return {boolean} True or false
 */
org.ametys.dao.content.Content.prototype.hasRight = function(rightId)
{
	return this._rights[rightId] == true;
}

/**
 * Add all the attributes to the given map
 * @param {Map<String,Object>} map The non null map
 */
org.ametys.dao.content.Content.prototype.toMap = function(map)
{
	map['id'] = this.getId(); 
	map['title'] = this.getTitle(); 
	map['path'] = this.getPath(); 
	map['lang'] = this.getLang(); 
	map['name'] = this.getName(); 
	map['type'] = this.getType(); 
	map['workflowName'] = this.getWorkflowName(); 
	map['isModifiable'] = this.isModifiable() ? "true" : "false";
   
	var rights = [];
	for (var right in this.getRights())
	{
		rights.push (right);
	}
	map['rights'] = rights;
}

/**
 * @private
 * @param {String} s A list of comma separated long
 * @return {long[]} The array of long. Never null
 */
org.ametys.dao.content.Content._convertStringToArray = function(s)
{
	if (s == null)
	{
		return [];
	}
	
	// remove unknown caracters
	s = s.replace(/[^0-9,]/g, '');
	
	var ids = s.split(",");
	
	for (var j = 0; j < ids.length; j++)
	{
		ids[j] = parseInt(ids[j]);
	}
	
	return ids;
}
