/*
 *  Copyright 2010 Anyware Services
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 */

Ext.namespace('org.ametys.ribbon.control');

/**
 * An element of the ribbon. This is an abstract class.
 * @param {String} id The unique element identifier
 * @param {Map<id, value>} configuration The configuration of the element instance.
 * @param {String} The name of the plugin where the element was declared
 * @class This interface is used for ribbon element (tab, control...) and any client side
 * @constructor
 */
org.ametys.ribbon.control.RibbonElement = function(id, configuration, pluginName)
{
	this._id = id;
	this._configuration = configuration;
	this._pluginName = pluginName;
}

/**
 * @property {String} _id The identifier of the element.
 * @private
 */
org.ametys.ribbon.control.RibbonElement.prototype._id;
/**
 * Get the identifier of the element.
 * @return {String} The unique identifier of the control.
 */
org.ametys.ribbon.control.RibbonElement.prototype.getId = function()
{
	return this._id;
}

/**
 * @private
 * @property {String} _pluginName The name of the plugin that declares the element
 */
org.ametys.ribbon.control.RibbonElement.prototype._pluginName;
/**
 * Get the plugin name that declared the element
 * @return {String} The name of the plugin that declared this element. Can not be null or empty.
 */
org.ametys.ribbon.control.RibbonElement.prototype.getPluginName = function()
{
	return this._pluginName;
}

/**
 * @private
 * @property {Map<id, value>} _configuration The configuration of the element instance.
 */
org.ametys.ribbon.control.RibbonElement.prototype._configuration;
/**
 * Get the configuration used at creation time. 
 * @return {Map<id, value>} The configuration of the control instance. Can not be null.
 */
org.ametys.ribbon.control.RibbonElement.prototype.getConfiguration = function()
{
	return this._configuration;
}

/**
 * This method is called when the configuration has changed. The reponse is converted and then the configurationChanged method is called
 * @param {Object} reponse The xml result
 * @param {String[]} arguments The argument given at call. null in this cas.
 * @private
 */
org.ametys.ribbon.control.RibbonElement.prototype._configurationChanged = function(response, arguments)
{
	if (org.ametys.servercomm.ServerComm.handleBadResponse("Une erreur est survenue : l'élement '" + this._id + "' n'a pas été mis à jour correctement", response, "org.ametys.ribbon.control.RibbonElement"))
	{
		if (typeof this.configurationError == "function")
		{
			this.configurationError(response);
		}
		return;
	}
	
	var configuration = {};
	
	var nodes = response.selectNodes("state/*/param");
	for (var n=0; n < nodes.length; n++)
	{
		var node = nodes[n];
		
		var name = node.getAttribute("name");
		var value = node[org.ametys.servercomm.ServerComm.xmlTextContent];
		
		configuration[name] = value;
	}
	
	this.configurationChanged(configuration)
}
/**
 * This method is called when the configuration has changed. This method is the callback of the RibbonManager.refreshState method. You must implement this method.
 * @param {Map<id, value>} configuration The configuration sent by the server
 */
org.ametys.ribbon.control.RibbonElement.prototype.configurationChanged = function(configuration)
{
	throw "RibbonElement.configureChanged has to be implemented";
}

